__author__ = "saeedamen"

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""
Shows how to use finmarketpy to total return indices for FX vanilla options 
(uses FinancePy underneath), so we can  see the historical P&L from for 
example, rolling a 1M call option etc.

Note, you will need to have a Bloomberg terminal (with blpapi Python library) 
to download the FX market data in order to generate the FX option prices, 
which are used underneath (FX spot, FX forwards, FX implied volatility quotes 
and deposit rates)
"""

import pandas as pd

# For plotting
from chartpy import Chart, Style

# For loading market data
from findatapy.market import Market, MarketDataGenerator, MarketDataRequest

from findatapy.timeseries import Filter, Calculations, RetStats

from findatapy.util.loggermanager import LoggerManager

from finmarketpy.curve.fxoptionscurve import FXOptionsCurve
from finmarketpy.curve.volatility.fxvolsurface import FXVolSurface
from finmarketpy.curve.volatility.fxoptionspricer import FXOptionsPricer

try:
    from finaddpy.market import CachedMarketDataGenerator as MarketDataGenerator
except:
    pass


logger = LoggerManager().getLogger(__name__)

chart = Chart(engine="plotly")

market = Market(market_data_generator=MarketDataGenerator())

# Choose run_example = 0 for everything
# run_example = 1 - create total return index AUDUSD 1M long calls (and 
# separately long puts) over 2008 financial crisis and further
# run_example = 2 - create total return index USDJPY 1W short straddles 
# over a long sample

run_example = 0


def prepare_indices(cross, df_option_tot=None, df_option_tc=None,
                    df_spot_tot=None):
    df_list = []

    if df_option_tot is not None:
        df_list.append(
            pd.DataFrame(df_option_tot[cross + "-option-tot.close"]))
        df_list.append(
            pd.DataFrame(df_option_tot[cross + "-option-delta-tot.close"]))
        df_list.append(
            pd.DataFrame(df_option_tot[cross + "-delta-pnl-index.close"]))

    if df_option_tc is not None:
        df_list.append(
            pd.DataFrame(df_option_tc[cross + "-option-tot-with-tc.close"]))
        df_list.append(pd.DataFrame(
            df_option_tc[cross + "-option-delta-tot-with-tc.close"]))
        df_list.append(pd.DataFrame(
            df_option_tc[cross + "-delta-pnl-index-with-tc.close"]))

    if df_spot_tot is not None:
        df_list.append(df_spot_tot)

    df = calculations.join(df_list, how="outer").fillna(method="ffill")

    return calculations.create_mult_index_from_prices(df)


if __name__ == "__main__":

    # Fetch market data for pricing AUDUSD options in 2007 (ie. FX spot,
    # FX forwards, FX deposits and FX vol quotes)
    # Construct volatility surface using FinancePy library underneath,
    # using polynomial interpolation
    # Enters a long 1M call, and MTM every day, and at expiry rolls into another long 1M call
    if run_example == 1 or run_example == 0:
        # Warning make sure you choose dates, where there is full vol surface!
        # If points are missing interpolation
        # will fail
        start_date = "01 Jan 2007";
        finish_date = "31 Dec 2020"  # Use smaller window for quicker execution

        cross = "AUDUSD"
        fx_options_trading_tenor = "1M"

        # Download the whole all market data for AUDUSD for pricing options
        # (FX vol surface + spot + FX forwards + depos)
        md_request = MarketDataRequest(start_date=start_date,
                                       finish_date=finish_date,
                                       data_source="bloomberg", cut="BGN",
                                       category="fx-vol-market",
                                       tickers=cross,
                                       fx_vol_tenor=["1W", "1M", "3M"],
                                       cache_algo="cache_algo_return",
                                       base_depos_currencies=[cross[0:3],
                                                              cross[3:6]])

        df_vol_market = market.fetch_market(md_request)
        df_vol_market = df_vol_market.fillna(method="ffill")

        # Remove New Year"s Day and Christmas
        df_vol_market = Filter().filter_time_series_by_holidays(df_vol_market,
                                                                cal="FX")

        # Get a total return index for trading spot
        # This way we can take into account carry when calculating delta
        # hedging P&L
        md_request = MarketDataRequest(start_date=start_date,
                                       finish_date=finish_date,
                                       data_source="bloomberg", cut="NYC",
                                       category="fx-tot",
                                       tickers=cross,
                                       cache_algo="cache_algo_return")

        df_tot = market.fetch_market(md_request)

        df_vol_market = df_vol_market.join(df_tot, how="left")
        df_vol_market = df_vol_market.fillna(method="ffill")

        # We want to roll long 1M ATM call at expiry
        # We"ll mark to market the option price through the month by
        # interpolating between 1W and 1M (and using whole vol curve
        # at each tenor)
        fx_options_curve = FXOptionsCurve(
            fx_options_trading_tenor=fx_options_trading_tenor,
            roll_days_before=0,
            roll_event="expiry-date",
            roll_months=1,
            fx_options_tenor_for_interpolation=["1W", "1M"],
            strike="atm",
            contract_type="european-call",
            depo_tenor_for_option="1M",
            position_multiplier=1.0,
            tot_label="tot",
            output_calculation_fields=True)

        # Let"s trade a long 1M call, and we roll at expiry
        df_cuemacro_option_call_tot = fx_options_curve.construct_total_return_index(
            cross, df_vol_market)

        # Add transaction costs to the option index (bid/ask bp for the option
        # premium and spot FX)
        df_cuemacro_option_call_tc = fx_options_curve.apply_tc_signals_to_total_return_index(
            cross, df_cuemacro_option_call_tot,
            option_tc_bp=5, spot_tc_bp=2)

        # Let"s trade a long 1M OTM put, and we roll at expiry
        df_cuemacro_option_put_tot = fx_options_curve.construct_total_return_index(
            cross, df_vol_market, contract_type="european-put",
            strike="10d-otm", position_multiplier=1.0)

        # Add transaction costs to the option index (bid/ask bp for the option
        # premium and spot FX)
        df_cuemacro_option_put_tc = fx_options_curve.apply_tc_signals_to_total_return_index(
            cross, df_cuemacro_option_put_tot,
            option_tc_bp=5, spot_tc_bp=2)

        # Get total returns for spot
        df_bbg_tot = df_tot  # from earlier!
        df_bbg_tot.columns = [x + "-bbg" for x in df_bbg_tot.columns]

        # Calculate a hedged portfolio of spot + 2*options (can we reduce drawdowns?)
        calculations = Calculations()

        ret_stats = RetStats()

        df_hedged = calculations.join(
            [df_bbg_tot[cross + "-tot.close-bbg"].to_frame(),
             df_cuemacro_option_put_tc[
                 cross + "-option-tot-with-tc.close"].to_frame()], how="outer")
        df_hedged = df_hedged.fillna(method="ffill")
        df_hedged = df_hedged.pct_change()

        df_hedged["Spot + 2*option put hedge"] = df_hedged[
                                                     cross + "-tot.close-bbg"] + \
                                                 df_hedged[
                                                     cross + "-option-tot-with-tc.close"]

        df_hedged.columns = RetStats(returns_df=df_hedged,
                                     ann_factor=252).summary()

        # Plot everything

        # P&L from call
        chart.plot(calculations.create_mult_index_from_prices(
            prepare_indices(cross=cross,
                            df_option_tot=df_cuemacro_option_call_tot,
                            df_option_tc=df_cuemacro_option_call_tc,
                            df_spot_tot=df_bbg_tot)))

        # P&L from put option, put option + TC and total returns from spot
        chart.plot(calculations.create_mult_index_from_prices(
            prepare_indices(cross=cross,
                            df_option_tot=df_cuemacro_option_put_tot,
                            df_option_tc=df_cuemacro_option_put_tc,
                            df_spot_tot=df_bbg_tot)))

        # P&L from put option + TC and total returns from spot
        chart.plot(calculations.create_mult_index_from_prices(
            prepare_indices(cross=cross,
                            df_option_tc=df_cuemacro_option_put_tc,
                            df_spot_tot=df_bbg_tot)))

        # P&L for total returns from spot and total returns from
        # + 2*put option + TC (ie. hedged portfolio)
        chart.plot(calculations.create_mult_index(df_hedged))

        # Plot delta from put option
        chart.plot(df_cuemacro_option_put_tot[cross + "-delta.close"])

    # Fetch market data for pricing EURUSD options from 2006-2020 (ie.
    # FX spot, FX forwards, FX deposits and FX vol quotes)
    # Construct volatility surface using FinancePy library underneath,
    # using polynomial interpolation
    #  Enters a short 1W straddle, and MTM every day, and at expiry rolls
    #  into another 1W straddle
    if run_example == 2 or run_example == 0:
        # Warning make sure you choose dates, where there is full vol surface!
        # If vol points in the tenors you are looking at
        # are missing then interpolation will fail (or if eg. spot data is missing etc.)
        start_date = "08 Mar 2007";
        finish_date = "31 Dec 2020"  # Monday
        # start_date = "09 Mar 2007"; finish_date = "31 Dec 2014"
        # start_date = "04 Jan 2006"; finish_date = "31 Dec 2008"
        # start_date = "01 Jan 2007"; finish_date = "31 Dec 2007" # Use smaller window for quicker execution

        cross = "USDJPY"
        fx_options_trading_tenor = "1W"  # Try changing between 1W and 1M!

        # Download the whole all market data for USDJPY for pricing options
        # (FX vol surface + spot + FX forwards + depos)
        md_request = MarketDataRequest(start_date=start_date,
                                       finish_date=finish_date,
                                       data_source="bloomberg", cut="10AM",
                                       category="fx-vol-market",
                                       tickers=cross,
                                       fx_vol_tenor=["1W", "1M"],
                                       base_depos_tenor=["1W", "1M"],
                                       cache_algo="cache_algo_return",
                                       base_depos_currencies=[cross[0:3],
                                                              cross[3:6]])

        df = market.fetch_market(md_request)

        # Fill data for every workday and use weekend calendar (note: this is
        # a bit of a fudge, filling down)
        # CHECK DATA isn"t missing at start of series
        df = df.resample("B").last().fillna(method="ffill")
        df = df[
            df.index >= "09 Mar 2007"]  # Try starting on a different day of
        # the week & see how it impact P&L?
        cal = "WKD"

        # Remove New Year"s Day and Christmas
        # df = Filter().filter_time_series_by_holidays(df, cal="FX")

        # We want to roll a short 1W option at expiry
        # If we select longer dates, it will mark to market the price through
        # the month by interpolating between eg. 1W and 1M
        # (and using whole vol curve at each tenor)
        fx_options_curve = FXOptionsCurve(
            fx_options_trading_tenor=fx_options_trading_tenor,
            roll_days_before=0,
            roll_event="expiry-date",
            roll_months=1,  # This is ignored if we roll on expiry date
            fx_options_tenor_for_interpolation=["1W", "1M"],
            strike="atm",
            contract_type="european-straddle",
            position_multiplier=-1.0,
            # +1.0 for long options, -1.0 for short options
            output_calculation_fields=True,
            freeze_implied_vol=True,
            cal=cal,
            cum_index="mult")

        # Let"s trade a short straddle, and we roll at expiry
        df_cuemacro_option_straddle_tot = fx_options_curve.construct_total_return_index(
            cross, df,
            depo_tenor_for_option=fx_options_trading_tenor)

        # Add transaction costs to the option index (bid/ask bp for the option
        # premium and spot FX)
        # Have wider spread for straddle (because adding call + put)
        df_cuemacro_option_straddle_tc = fx_options_curve.apply_tc_signals_to_total_return_index(
            cross, df_cuemacro_option_straddle_tot,
            option_tc_bp=10, spot_tc_bp=2)

        # Get total returns for spot
        md_request.abstract_curve = None

        # Get Bloomberg calculated total return indices (for spot)
        md_request.category = "fx-tot"
        md_request.cut = "NYC"

        df_bbg_tot = market.fetch_market(md_request)
        df_bbg_tot.columns = [x + "-bbg" for x in df_bbg_tot.columns]

        calculations = Calculations()

        df_index = calculations.create_mult_index_from_prices(
            prepare_indices(cross=cross,
                            df_option_tc=df_cuemacro_option_straddle_tc,
                            df_spot_tot=df_bbg_tot))

        from finmarketpy.economics.quickchart import QuickChart

        QuickChart(engine="plotly").plot_chart_with_ret_stats(
              df=df_index,
              plotly_plot_mode="offline_html",
              scale_factor=-1.5)



__author__ = "saeedamen"

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
Shows how to use finmarketpy to create total return indices for FX spot (ie. 
calculates spot returns + carry returns)
"""

import pandas as pd

# For plotting
from chartpy import Chart, Style

# For loading market data
from findatapy.market import Market, MarketDataGenerator, MarketDataRequest
from findatapy.timeseries import Calculations

from findatapy.util.loggermanager import LoggerManager

logger = LoggerManager().getLogger(__name__)

chart = Chart(engine="plotly")
market = Market(market_data_generator=MarketDataGenerator())
calculations = Calculations()

# Choose run_example = 0 for everything
# run_example = 1 - create daily total return indices from FX spot data +
# deposit for AUDJPY, and compare
# run_example = 2 - create intraday total return indices from FX spot data +
# deposit for GBPUSD, and compare with daily

run_example = 0

from finmarketpy.curve.fxspotcurve import FXSpotCurve

# Create total return indices plot for AUDJPY (from perspective of a USD investor)
# Compare with AUDJPY FX spot and BBG constructed AUDJPY total return indices
if run_example == 1 or run_example == 0:
    # Get AUDJPY total returns from perspective of USD investor (via AUDUSD &
    # JPYUSD and AUD, USD & JPY overnight deposit rates)
    md_request = MarketDataRequest(start_date="01 Jan 1999",
                                   finish_date="01 Dec 2020",
                                   data_source="bloomberg", cut="NYC",
                                   category="fx",
                                   tickers=["AUDJPY"],
                                   cache_algo="cache_algo_return",
                                   abstract_curve=FXSpotCurve(
                                       construct_via_currency="USD",
                                       depo_tenor="ON"))

    df_tot = market.fetch_market(md_request=md_request)
    df_tot.columns = [x + "-tot-cuemacro" for x in df_tot.columns]

    # Get spot data
    md_request.abstract_curve = None

    df_spot = market.fetch_market(md_request=md_request)
    df_spot.columns = [x + "-spot" for x in df_spot.columns]

    # Get Bloomberg calculated total return indices (for spot)
    md_request.category = "fx-tot"

    df_bbg_tot = market.fetch_market(md_request)
    df_bbg_tot.columns = [x + "-bbg" for x in df_bbg_tot.columns]

    # Get Bloomberg calculated total return indices (for 1M forwards rolled)
    md_request.category = "fx-tot-forwards"

    df_bbg_tot_forwards = market.fetch_market(md_request)
    df_bbg_tot_forwards.columns = [x + "-bbg" for x in
                                   df_bbg_tot_forwards.columns]

    # Combine into a single data frame and plot, we note that the Cuemacro
    # constructed indices track the Bloomberg indices relatively well (both
    # from spot and 1M forwards). Also note the large difference with spot indices
    # CAREFUL to fill down, before reindexing because 1M forwards indices are
    # likely to have different publishing dates
    df = calculations.join([df_tot, df_bbg_tot, df_spot, df_bbg_tot_forwards],
                           how="outer").fillna(method="ffill")
    df = calculations.create_mult_index_from_prices(df)

    chart.plot(df)

# Create total return indices plot for GBPUSD with intraday and daily data
# (from perspective of a USD investor)
# Compare intraday and daily total return indices
if run_example == 2 or run_example == 0:
    import pytz

    # Get GBPUSD total returns from perspective of USD investor (via GBP and USD rates)
    md_request = MarketDataRequest(start_date="01 Jan 2019",
                                   finish_date="01 Jul 2019",
                                   data_source="bloomberg", cut="NYC",
                                   category="fx",
                                   tickers=["GBPUSD"],
                                   cache_algo="cache_algo_return",
                                   abstract_curve=FXSpotCurve(
                                       construct_via_currency="USD",
                                       depo_tenor="ON"))

    df_tot = market.fetch_market(md_request=md_request)
    df_tot.columns = [x + "-tot-cuemacro" for x in df_tot.columns]
    df_tot = df_tot.tz_localize(pytz.utc)
    df_tot.index = df_tot.index + pd.Timedelta(
        hours=22)  # Roughly NY close 2200 GMT

    md_request.abstract_curve = None

    # Get intraday spot data
    md_request.freq = "tick"
    md_request.data_source = "dukascopy"

    df_intraday_spot = market.fetch_market(md_request=md_request)
    df_intraday_spot = pd.DataFrame(
        df_intraday_spot.resample("1min").last().dropna())

    # Get Bloomberg calculated total return indices (for spot)
    md_request.category = "fx-tot"
    md_request.freq = "daily"
    md_request.data_source = "bloomberg"

    df_bbg_tot = market.fetch_market(md_request)
    df_bbg_tot.columns = [x + "-bbg" for x in df_bbg_tot.columns]
    df_bbg_tot = df_bbg_tot.tz_localize(pytz.utc)
    df_bbg_tot.index = df_bbg_tot.index + pd.Timedelta(
        hours=22)  # Roughly NY close 2200 GMT

    md_request = MarketDataRequest(start_date="01 Jan 2019",
                                   finish_date="01 Jul 2019",
                                   data_source="bloomberg", cut="NYC",
                                   category="base-depos",
                                   tickers=["GBPON", "USDON"],
                                   cache_algo="cache_algo_return")

    # Join daily deposit data with intraday spot data
    # OK to fill down, because deposit data isn"t very volatile
    df_deposit_rates = market.fetch_market(md_request).tz_localize(pytz.utc)

    df_intraday_market = df_intraday_spot.join(df_deposit_rates, how="left")
    df_intraday_market = df_intraday_market.fillna(method="ffill").fillna(
        method="bfill")

    df_intraday_tot = FXSpotCurve().construct_total_return_index(
        "GBPUSD", df_intraday_market, depo_tenor="ON")

    df_intraday_spot.columns = [x + "-intraday-spot" for x in
                                df_intraday_spot.columns]
    df_intraday_tot.columns = [x + "-intraday-tot" for x in
                               df_intraday_spot.columns]

    # Combine into a single data frame and plot
    df = calculations.join(
        [df_bbg_tot, df_tot, df_intraday_tot, df_intraday_spot],
        how="outer").fillna(method="ffill")
    df = calculations.create_mult_index_from_prices(df)

    chart.plot(df)



__author__ = "saeedamen"

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
Shows how to use finmarketpy to create total return indices for FX forwards 
with appropriate roll rules
"""

import pandas as pd

# For plotting
from chartpy import Chart, Style

# For loading market data
from findatapy.market import Market, MarketDataGenerator, MarketDataRequest
from findatapy.timeseries import Calculations

from findatapy.util.loggermanager import LoggerManager

logger = LoggerManager().getLogger(__name__)

chart = Chart(engine="plotly")
market = Market(market_data_generator=MarketDataGenerator())
calculations = Calculations()

# Choose run_example = 0 for everything
# run_example = 1 - creating USDBRL total return index rolling forwards and 
# compare with BBG indices
# run_example = 2 - creating AUDJPY (via AUDUSD and JPYUSD) total return index 
# rolling forwards & compare with BBG indices

run_example = 0

from finmarketpy.curve.fxforwardscurve import FXForwardsCurve

###### Create total return indices plot for USDBRL using forwards
# We shall be using USDBRL 1M forward contracts and rolling them 5 business
# days before month end
if run_example == 1 or run_example == 0:
    cross = "USDBRL"

    # Download more tenors
    fx_forwards_tenors = ["1W", "1M", "2M", "3M"]

    # Get USDBRL data for spot, forwards + depos
    md_request = MarketDataRequest(start_date="02 Jan 2007",
                                   finish_date="01 Jun 2007",
                                   data_source="bloomberg", cut="NYC",
                                   category="fx-forwards-market",
                                   tickers=cross,
                                   fx_forwards_tenor=fx_forwards_tenors,
                                   base_depos_currencies=[cross[0:3],
                                                          cross[3:6]],
                                   cache_algo="cache_algo_return")

    # In case any missing values fill down (particularly can get this for NDFs)
    df_market = market.fetch_market(md_request=md_request).fillna(
        method="ffill")

    fx_forwards_curve = FXForwardsCurve()

    # Let"s trade a 1M forward, and we roll 5 business days (based on both
    # base + terms currency holidays)
    # before month end
    df_cuemacro_tot_1M = fx_forwards_curve.construct_total_return_index(
        cross,
        df_market,
        fx_forwards_trading_tenor="1M",
        roll_days_before=5,
        roll_event="month-end",
        roll_months=1,
        fx_forwards_tenor_for_interpolation=fx_forwards_tenors,
        output_calculation_fields=True)

    df_cuemacro_tot_1M.columns = [
        x.replace("forward-tot", "forward-tot-1M-cuemacro") for x in
        df_cuemacro_tot_1M.columns]

    # Now do a 3M forward, and we roll 5 business days before end of quarter(
    # based on both base + terms currency holidays)
    # before month end
    df_cuemacro_tot_3M = fx_forwards_curve.construct_total_return_index(
        cross,
        df_market,
        fx_forwards_trading_tenor="3M",
        roll_days_before=5,
        roll_event="month-end",
        roll_months=3,
        fx_forwards_tenor_for_interpolation=fx_forwards_tenors,
        output_calculation_fields=True)

    df_cuemacro_tot_3M.columns = [
        x.replace("forward-tot", "forward-tot-3M-cuemacro") for x in
        df_cuemacro_tot_3M.columns]

    # Get spot data
    md_request.abstract_curve = None
    md_request.category = "fx"

    df_spot = market.fetch_market(md_request=md_request)
    df_spot.columns = [x + "-spot" for x in df_spot.columns]

    # Get Bloomberg calculated total return indices (for spot)
    md_request.category = "fx-tot"

    df_bbg_tot = market.fetch_market(md_request)
    df_bbg_tot.columns = [x + "-bbg" for x in df_bbg_tot.columns]

    # Get Bloomberg calculated total return indices (for 1M forwards rolled)
    md_request.category = "fx-tot-forwards"

    df_bbg_tot_forwards = market.fetch_market(md_request)
    df_bbg_tot_forwards.columns = [x + "-bbg" for x in
                                   df_bbg_tot_forwards.columns]

    # Combine into a single data frame and plot, we note that the Cuemacro
    # constructed indices track the Bloomberg
    # indices relatively well (both from spot and forwards). Also note the large
    # difference with spot indices
    # CAREFUL to fill down, before reindexing because forwards indices are
    # likely to have different publishing dates
    df = calculations.join([pd.DataFrame(
        df_cuemacro_tot_1M[cross + "-forward-tot-1M-cuemacro.close"]),
                            pd.DataFrame(df_cuemacro_tot_3M[
                                             cross + "-forward-tot-3M-cuemacro.close"]),
                            df_bbg_tot, df_spot, df_bbg_tot_forwards],
                           how="outer").fillna(method="ffill")

    df = calculations.create_mult_index_from_prices(df)

    chart.plot(df)

###### Create total return indices plot for AUDJPY using the underlying USD
# legs (ie. AUDUSD & JPYUSD)
if run_example == 2 or run_example == 0:
    cross = "AUDJPY"

    # Download more tenors
    fx_forwards_tenors = ["1W", "1M", "2M", "3M"]

    # Parameters for how to construct total return indices, and the rolling rule
    # 1M forward contract, and roll it 5 working days before month end
    # We"ll be constructing our total return index from AUDUSD and JPYUSD
    fx_forwards_curve = FXForwardsCurve(
        fx_forwards_trading_tenor="1M",
        roll_days_before=5,
        roll_event="month-end",
        construct_via_currency="USD",
        fx_forwards_tenor_for_interpolation=fx_forwards_tenors,
        roll_months=1,
        output_calculation_fields=True)

    # Get AUDJPY (AUDUSD and JPYUSD) data for spot, forwards + depos and also
    # construct the total returns forward index
    md_request = MarketDataRequest(start_date="02 Jan 2007",
                                   finish_date="01 Jun 2007",
                                   data_source="bloomberg", cut="NYC",
                                   category="fx",
                                   tickers=cross,
                                   fx_forwards_tenor=fx_forwards_tenors,
                                   base_depos_currencies=[cross[0:3],
                                                          cross[3:6]],
                                   cache_algo="cache_algo_return",
                                   abstract_curve=fx_forwards_curve)

    # In case any missing values fill down (particularly can get this for NDFs)
    df_cuemacro_tot_1M = market.fetch_market(md_request=md_request).fillna(
        method="ffill")

    fx_forwards_curve = FXForwardsCurve()

    df_cuemacro_tot_1M.columns = [
        x.replace("forward-tot", "forward-tot-1M-cuemacro") for x in
        df_cuemacro_tot_1M.columns]

    # Get spot data
    md_request.abstract_curve = None
    md_request.category = "fx"

    df_spot = market.fetch_market(md_request=md_request)
    df_spot.columns = [x + "-spot" for x in df_spot.columns]

    # Get Bloomberg calculated total return indices (for spot)
    md_request.category = "fx-tot"

    df_bbg_tot = market.fetch_market(md_request)
    df_bbg_tot.columns = [x + "-bbg" for x in df_bbg_tot.columns]

    # Get Bloomberg calculated total return indices (for 1M forwards rolled)
    md_request.category = "fx-tot-forwards"

    df_bbg_tot_forwards = market.fetch_market(md_request)
    df_bbg_tot_forwards.columns = [x + "-bbg" for x in
                                   df_bbg_tot_forwards.columns]

    # Combine into a single data frame and plot, we note that the Cuemacro
    # constructed indices track the Bloomberg
    # indices relatively well (both from spot and forwards). Also note the
    # large difference with spot indices
    # CAREFUL to fill down, before reindexing because forwards indices are
    # likely to have different publishing dates
    df = calculations.join([pd.DataFrame(
        df_cuemacro_tot_1M[cross + "-forward-tot-1M-cuemacro.close"]),
                            df_bbg_tot, df_spot, df_bbg_tot_forwards],
                           how="outer").fillna(method="ffill")

    df = calculations.create_mult_index_from_prices(df)

    chart.plot(df)



__author__ = 'saeedamen'  # Saeed Amen

#
# Copyright 2016-2020 Cuemacro - https://www.cuemacro.com / @cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

import numpy as np
import pandas as pd

from numba import guvectorize
from findatapy.timeseries import Calendar
from findatapy.util import LoggerManager

from finmarketpy.util.marketconstants import MarketConstants
from finmarketpy.curve.abstractpricer import AbstractPricer
from finmarketpy.curve.rates.fxforwardspricer import FXForwardsPricer

from financepy.utils.date import Date
from financepy.models.black_scholes import BlackScholes
from financepy.products.fx.fx_vanilla_option import FXVanillaOption
from financepy.utils.global_types import OptionTypes

# from financepy.products.fx.FinFXMktConventions import *

market_constants = MarketConstants()


class FXOptionsPricer(AbstractPricer):
    """Prices various vanilla FX options, using FinancePy underneath.
    """

    def __init__(self, fx_vol_surface=None,
                 premium_output=market_constants.fx_options_premium_output,
                 delta_output=market_constants.fx_options_delta_output):

        self._calendar = Calendar()
        self._fx_vol_surface = fx_vol_surface
        self._fx_forwards_pricer = FXForwardsPricer()
        self._premium_output = premium_output
        self._delta_output = delta_output

    def price_instrument(self,
                         cross,
                         horizon_date,
                         strike,
                         expiry_date=None,
                         vol=None, notional=1000000,
                         contract_type='european-call',
                         tenor=None,
                         fx_vol_surface=None,
                         premium_output=None,
                         delta_output=None,
                         depo_tenor=None,
                         use_atm_quoted=False,
                         return_as_df=True):
        """Prices FX options for horizon dates/expiry dates given by the user from FX spot rates, FX volatility surface
        and deposit rates.

        Parameters
        ----------
        cross : str
            Currency pair

        horizon_date : DateTimeIndex
            Horizon dates for options

        strike : np.ndarray, float or str
            Strike of option

            eg. "atm" - at-the-money
            eg. "atmf" - at-the-money forward
            eg. "atms" - at-the-money spot
            eg. "25d-otm" - out-of-the-money 25d
            eg. "10d-otm" - out-of-the-money 10d

        expiry_date : DateTimeIndex (optional)
            Expiry dates for options

        vol : np.ndarray (optional)
            Umplied vol for options

        notional : float
            Notional in base currency of the option

        contract_type : str
            What type of option are we pricing?

            eg. "european-call"

        tenor : str (optional)
            Tenor of option

        fx_vol_surface : FXVolSurface
            Interpolates FX vol surface

        premium_output : str
            'pct-for' (in base currency pct) or 'pct-dom' (in terms currency pct)

        delta_output : bool
            Also output delta of options

        depo_tenor : str
            Tenor of the deposit to use in the option pricing

        use_atm_quoted : bool
            True - takes the direct market quote
            False - uses interpolated version

        return_as_df : bool
            True - returns output as DataFrame
            False - returns output as np.ndarray

        Returns
        -------
        DataFrame
        """

        # if market_df is None: market_df = self._market_df
        if fx_vol_surface is None: fx_vol_surface = self._fx_vol_surface
        if premium_output is None: premium_output = self._premium_output
        if delta_output is None: delta_output = self._delta_output

        logger = LoggerManager().getLogger(__name__)

        field = fx_vol_surface._field

        # Make horizon date and expiry date pandas DatetimeIndex
        if isinstance(horizon_date, pd.Timestamp):
            horizon_date = pd.DatetimeIndex([horizon_date])
        else:
            horizon_date = pd.DatetimeIndex(horizon_date)

        if expiry_date is not None:
            if isinstance(expiry_date, pd.Timestamp):
                expiry_date = pd.DatetimeIndex([expiry_date])
            else:
                expiry_date = pd.DatetimeIndex(expiry_date)
        else:
            expiry_date = self._calendar.get_expiry_date_from_horizon_date(
                horizon_date, tenor, cal=cross)

        # If the strike hasn't been supplied need to work this out
        if not (isinstance(strike, np.ndarray)):
            old_strike = strike

            if isinstance(strike, str):
                strike = np.empty(len(horizon_date), dtype=object)
            else:
                strike = np.empty(len(horizon_date))

            strike.fill(old_strike)

        # If the vol hasn't been supplied need to work this out
        if not (isinstance(vol, np.ndarray)):

            if vol is None:
                vol = np.nan

            old_vol = vol

            vol = np.empty(len(horizon_date))
            vol.fill(old_vol)

        option_values = np.zeros(len(horizon_date))
        spot = np.zeros(len(horizon_date))
        delta = np.zeros(len(horizon_date))
        intrinsic_values = np.zeros(len(horizon_date))

        def _price_option(contract_type_, contract_type_fin_):
            for i in range(len(expiry_date)):
                built_vol_surface = False

                # If we have a "key strike" need to fit the vol surface
                if isinstance(strike[i], str):
                    if not (built_vol_surface):
                        fx_vol_surface.build_vol_surface(horizon_date[i])
                        fx_vol_surface.extract_vol_surface(
                            num_strike_intervals=None)

                        built_vol_surface = True

                    # Delta neutral strike/or whatever strike is quoted as ATM
                    # usually this is ATM delta neutral strike, but can sometimes be ATMF for some Latam
                    # Take the vol directly quoted, rather than getting it from building vol surface
                    if strike[i] == "atm":
                        strike[i] = fx_vol_surface.get_atm_strike(tenor)

                        if use_atm_quoted:
                            vol[i] = fx_vol_surface.get_atm_quoted_vol(
                                tenor) / 100.0
                        else:
                            vol[i] = fx_vol_surface.get_atm_vol(
                                tenor) / 100.0  # interpolated
                    elif strike[i] == "atms":
                        strike[
                            i] = fx_vol_surface.get_spot()  # Interpolate vol later
                    elif strike[i] == "atmf":
                        # Quoted tenor, no need to interpolate
                        strike[i] = float(fx_vol_surface.get_all_market_data()[
                                              cross + ".close"][
                                              horizon_date[i]]) \
                                    + (float(
                            fx_vol_surface.get_all_market_data()[
                                cross + tenor + ".close"][horizon_date[i]]) \
                                       / self._fx_forwards_pricer.get_forwards_divisor(
                                    cross[3:6]))

                        # Interpolate vol later

                    # TODO: work on 25d and 10d strikes
                    elif strike[i] == "25d-otm":
                        if "call" in contract_type_:
                            strike[i] = fx_vol_surface.get_25d_call_strike(
                                tenor)

                            vol[i] = fx_vol_surface.get_25d_call_vol(
                                tenor) / 100.0
                        elif "put" in contract_type_:
                            strike[i] = fx_vol_surface.get_25d_put_strike(
                                tenor)
                            vol[i] = fx_vol_surface.get_25d_put_vol(
                                tenor) / 100.0

                    elif strike[i] == "10d-otm":
                        if "call" in contract_type_:
                            strike[i] = fx_vol_surface.get_10d_call_strike(
                                tenor)
                            vol[i] = fx_vol_surface.get_10d_call_vol(
                                tenor) / 100.0
                        elif "put" in contract_type_:
                            strike[i] = fx_vol_surface.get_10d_put_strike(
                                tenor)
                            vol[i] = fx_vol_surface.get_10d_put_vol(
                                tenor) / 100.0

                if not built_vol_surface:
                    try:
                        fx_vol_surface.build_vol_surface(horizon_date[i])
                    except:
                        logger.warn("Failed to build vol surface for " + str(
                            horizon_date) + ", won't be able to interpolate vol")
                    # fx_vol_surface.extract_vol_surface(num_strike_intervals=None)

                # If an implied vol hasn't been provided, interpolate that one, fit the vol surface (if hasn't already been
                # done)
                if np.isnan(vol[i]):

                    if tenor is None:
                        vol[
                            i] = fx_vol_surface.calculate_vol_for_strike_expiry(
                            strike[i], expiry_date=expiry_date[i], tenor=None)
                    else:
                        vol[
                            i] = fx_vol_surface.calculate_vol_for_strike_expiry(
                            strike[i], expiry_date=None, tenor=tenor)

                model = BlackScholes(float(vol[i]))

                logger.info(
                    "Pricing " + contract_type_ + " option, horizon date = " + str(
                        horizon_date[i]) + ", expiry date = "
                    + str(expiry_date[i]))

                option = FXVanillaOption(
                    self._findate(expiry_date[i]), strike[i],
                    cross, contract_type_fin_, notional, cross[0:3])

                spot[i] = fx_vol_surface.get_spot()

                """ FinancePy will return the value in the following dictionary for values
                    {"v": vdf,
                    "cash_dom": cash_dom,
                    "cash_for": cash_for,
                    "pips_dom": pips_dom,
                    "pips_for": pips_for,
                    "pct_dom": pct_dom,
                    "pct_for": pct_for,
                    "not_dom": notional_dom,
                    "not_for": notional_for,
                    "ccy_dom": self._domName,
                    "ccy_for": self._forName}
                """

                option_values[i] = option_values[i] + \
                                   option.value(self._findate(horizon_date[i]),
                                                spot[i],
                                                fx_vol_surface.get_dom_discount_curve(),
                                                fx_vol_surface.get_for_discount_curve(),
                                                model)[
                                       premium_output.replace("-", "_")]

                # TODO instrinsic value calculations need checking!
                if contract_type == "european-call":
                    intrinsic_values[i] = (max(spot[i] - strike[i], 0))
                elif contract_type == "european-put":
                    intrinsic_values[i] = (max(strike[i] - spot[i], 0))
                elif contract_type == "european-straddle" or contract_type == "european-strangle":
                    intrinsic_values[i] = (max(spot[i] - strike[i], 0) + max(
                        strike[i] - spot[i], 0))

                if "pct-for" in premium_output:
                    intrinsic_values[i] = intrinsic_values[i] / spot[i]

                # intrinsic_values[i] = intrinsic_values[i] + option.value(self._findate(expiry_date[i]),
                #                                 spot[i], fx_vol_surface.get_dom_discount_curve(),
                #                                 fx_vol_surface.get_for_discount_curve(),
                #                                 model)[premium_output.replace("-", "_")]

                """FinancePy returns this dictionary for deltas
                    {"pips_spot_delta": pips_spot_delta,
                    "pips_fwd_delta": pips_fwd_delta,
                    "pct_spot_delta_prem_adj": pct_spot_delta_prem_adj,
                    "pct_fwd_delta_prem_adj": pct_fwd_delta_prem_adj}
                """

                delta[i] = delta[i] + \
                           option.delta(self._findate(horizon_date[i]),
                                        spot[i],
                                        fx_vol_surface.get_dom_discount_curve(),
                                        fx_vol_surface.get_for_discount_curve(),
                                        model)[delta_output.replace('-', '_')]

        if contract_type == "european-call":
            contract_type_fin = OptionTypes.EUROPEAN_CALL

            _price_option(contract_type, contract_type_fin)
        elif contract_type == "european-put":
            contract_type_fin = OptionTypes.EUROPEAN_PUT

            _price_option(contract_type, contract_type_fin)
        elif contract_type == "european-straddle" or contract_type == "european-strangle":
            contract_type = "european-call"
            contract_type_fin = OptionTypes.EUROPEAN_CALL

            _price_option(contract_type, contract_type_fin)

            contract_type = "european-put"
            contract_type_fin = OptionTypes.EUROPEAN_PUT

            _price_option(contract_type, contract_type_fin)

        if return_as_df:
            option_prices_df = pd.DataFrame(index=horizon_date)

            option_prices_df[cross + "-option-price." + field] = option_values
            option_prices_df[cross + "." + field] = spot
            option_prices_df[cross + "-strike." + field] = strike
            option_prices_df[cross + "-vol." + field] = vol
            option_prices_df[cross + "-delta." + field] = delta
            option_prices_df[cross + ".expiry-date"] = expiry_date
            option_prices_df[
                cross + "-intrinsic-value." + field] = intrinsic_values

            return option_prices_df

        return option_values, spot, strike, vol, delta, expiry_date, intrinsic_values

    def get_day_count_conv(self, currency):
        if currency in market_constants.currencies_with_365_basis:
            return 365.0

        return 360.0

    def _findate(self, timestamp):

        return Date(timestamp.day, timestamp.month, timestamp.year,
                    hh=timestamp.hour, mm=timestamp.minute,
                    ss=timestamp.second)



__author__ = 'saeedamen'  # Saeed Amen

#
# Copyright 2016-2020 Cuemacro - https://www.cuemacro.com / @cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

import abc

class AbstractCurve(object):
    """Abstract class for creating total return indices and curves, which is for example implemented by FXSpotCurve
    and could be implemented by other asset classes.

    """

    @abc.abstractmethod
    def generate_key(self):
        return

    @abc.abstractmethod
    def fetch_continuous_time_series(self, md_request, market_data_generator):
        return

    @abc.abstractmethod
    def construct_total_returns_index(self):
        return



__author__ = "saeedamen"

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

"""
Shows how to use finmarketpy to process FX vol surfaces which have been interpolated (uses FinancePy underneath).

Note, you will need to have a Bloomberg terminal (with blpapi Python library) to download the FX market data in order
to plot these vol surface (FX spot, FX forwards, FX implied_vol volatility quotes and deposits)
"""

# For plotting
from chartpy import Chart, Style

# For loading market data
from findatapy.market import Market, MarketDataGenerator, MarketDataRequest

from findatapy.util.loggermanager import LoggerManager

from finmarketpy.curve.volatility.fxvolsurface import FXVolSurface

try:
    from finaddpy.market.cachedmarketdatagenerator import \
        CachedMarketDataGenerator as MarketDataGenerator
except:
    pass

logger = LoggerManager().getLogger(__name__)

chart = Chart(engine="plotly")
market = Market(market_data_generator=MarketDataGenerator())

# Choose run_example = 0 for everything
# run_example = 1 - plot GBPUSD 1W implied vol
# run_example = 2 - get GBPUSD vol surface for a date and plot interpolated vol surface and implied PDF
# run_example = 3 - do an animation of GBPUSD implied vol surface over this period
# run_example = 4 - get implied vol for a particular strike, interpolating the surface
# run_example = 5 - get USDJPY vol surface around US presidential election and plot

run_example = 0

###### Fetch market data for pricing GBPUSD FX options over Brexit vote (ie. FX spot, FX forwards, FX deposits and FX vol quotes)
###### Show how to plot ATM 1M implied_vol vol time series
if run_example == 1 or run_example == 0:
    # Download the whole all market data for GBPUSD for pricing options (vol surface)
    md_request = MarketDataRequest(start_date="01 May 2016",
                                   finish_date="01 Aug 2016",
                                   data_source="bloomberg", cut="LDN",
                                   category="fx-vol-market",
                                   tickers=["GBPUSD"],
                                   cache_algo="cache_algo_return")

    df = market.fetch_market(md_request)

    style = Style()

    style.title = "GBPUSD 1M Implied Vol"
    style.scale_factor = 3
    style.source = "Bloomberg"

    chart.plot(df["GBPUSDV1M.close"], style=style)

###### Fetch market data for pricing GBPUSD FX options over Brexit vote (ie. FX spot, FX forwards, FX deposits and FX vol quotes)
###### Construct volatility surface using FinancePy library underneath, using polynomial interpolation
if run_example == 2 or run_example == 0:
    horizon_date = "23 Jun 2016"
    cross = "GBPUSD"

    # Download the whole all market data for GBPUSD for pricing options (vol surface)
    md_request = MarketDataRequest(start_date=horizon_date,
                                   finish_date=horizon_date,
                                   data_source="bloomberg", cut="LDN",
                                   category="fx-vol-market",
                                   tickers=cross,
                                   cache_algo="cache_algo_return")

    df = market.fetch_market(md_request)

    fx_vol_surface = FXVolSurface(market_df=df, vol_function_type="BBG",
                                  asset=cross)

    fx_vol_surface.build_vol_surface(horizon_date)

    # Note for unstable vol surface dates (eg. over Brexit date) you may need to increase tolerance in FinancePy
    # FinFXVolSurface.buildVolSurface method to get it to fill, or choose different vol_function_type (eg. "CLARK5")
    df_vol_dict = fx_vol_surface.extract_vol_surface(low_K_pc=0.80,
                                                     high_K_pc=1.1)

    # Print out the various vol surface and data produced
    print(df_vol_dict["vol_surface_implied_pdf"])
    print(df_vol_dict["vol_surface_strike_space"])
    print(df_vol_dict["vol_surface_delta_space"])
    print(df_vol_dict["vol_surface_delta_space_exc_ms"])
    print(df_vol_dict["deltas_vs_strikes"])
    print(df_vol_dict["vol_surface_quoted_points"])

    # Plot vol surface in strike space (all interpolated)
    # x_axis = strike - index
    # y_axis = tenor - columns
    # z_axis = implied vol - values
    chart.plot(df_vol_dict["vol_surface_strike_space"].iloc[:, ::-1],
               chart_type="surface",
               style=Style(title="Plotting volatility in strike space"))

    # Plot vol surface in delta space (exc market strangle strikes)
    chart.plot(df_vol_dict["vol_surface_delta_space_exc_ms"].iloc[:, ::-1],
               chart_type="surface",
               style=Style(title="Plotting in delta space"))

    # Plot implied PDF in strike space (all interpolated)
    # x_axis = strike - index
    # y_axis = tenor - columns
    # z_axis = implied PDF - values
    chart.plot(df_vol_dict["vol_surface_implied_pdf"], chart_type="surface",
               style=Style(title="Plotting implied PDF in strike space"))

    # Plot the implied PDF for ON only versus strikes
    chart.plot(df_vol_dict["vol_surface_implied_pdf"]["ON"], chart_type="line",
               style=Style(
                   title="Plotting implied PDF in strike space ON around Brexit",
                   x_axis_range=[1.0, 1.8]))

###### Fetch market data for pricing GBPUSD FX options over Brexit vote (ie. FX spot, FX forwards, FX deposits and FX vol quotes)
###### Do animation for vol surface
if run_example == 3 or run_example == 0:
    # Download the whole all market data for GBPUSD for pricing options (vol surface)
    # Using LDN close data (CMPL)
    md_request = MarketDataRequest(start_date="01 Jun 2016",
                                   finish_date="30 Jul 2016",
                                   data_source="bloomberg", cut="LDN",
                                   category="fx-vol-market",
                                   tickers=["GBPUSD"],
                                   cache_algo="cache_algo_return")
    # 01 Jun 2016, 30 Jun 2016
    # 20 Jun 2016, 24 Jun 2016

    df = market.fetch_market(md_request)

    fx_vol_surface = FXVolSurface(market_df=df, asset="GBPUSD")

    animate_titles = []

    # Note for unstable vol surface dates (eg. over Brexit date) you may need to increase tolerance in FinancePy
    # FinFXVolSurface.buildVolSurface method to get it to fill - or just change dates and currency pair

    # Note this does take a few minutes, given it's fitting the vol surface for every date
    # TODO explore speeding up using Numba or similar
    vol_surface_dict, extremes_dict = fx_vol_surface.extract_vol_surface_across_dates(
        df.index,
        vol_surface_type="vol_surface_strike_space")

    animate_titles = [x.strftime("%d %b %Y") for x in vol_surface_dict.keys()]

    print(extremes_dict)

    # Plot vol surface animation in strike space (all interpolated)
    # x_axis = strike - index
    # y_axis = tenor - columns
    # z_axis = implied_vol vol - values
    style = Style(title="Plotting in strike space", animate_figure=True,
                  animate_titles=animate_titles)

    chart.plot(list(vol_surface_dict.values()), chart_type="surface",
               style=style)

###### Fetch market data for pricing GBPUSD FX options over Brexit vote (ie. FX spot, FX forwards, FX deposits and FX vol quotes)
###### Get implied_vol vol for specific strikes interpolating across surface
if run_example == 4 or run_example == 0:
    # Download the whole all market data for GBPUSD for pricing options (vol surface)
    md_request = MarketDataRequest(start_date="20 Jun 2016",
                                   finish_date="25 Jun 2016",
                                   data_source="bloomberg", cut="LDN",
                                   category="fx-vol-market",
                                   tickers=["GBPUSD"],
                                   cache_algo="cache_algo_return")

    df = market.fetch_market(md_request)

    fx_vol_surface = FXVolSurface(market_df=df, asset="GBPUSD")

    df_vol_surface_strike_space_list = []
    animate_titles = []

    fx_vol_surface.build_vol_surface("20 Jun 2016")

    # Get the implied_vol volatility for a specific strike (GBPUSD=1.4000 in the 1W tenor) for 20 Jun 2016
    vol_at_strike = fx_vol_surface.calculate_vol_for_strike_expiry(1.4000,
                                                                   tenor="1W")

    fx_vol_surface.build_vol_surface("23 Jun 2016")

    # Get the implied_vol volatility for a specific strike (GBPUSD=1.4000 in the 1W tenor) for 23 Jun 2016
    vol_at_strike = fx_vol_surface.calculate_vol_for_strike_expiry(1.4000,
                                                                   tenor="1W")

    print(vol_at_strike)

###### Fetch market data for pricing USDJPY FX options near 2020 US Presidential Election (03 Nov 2020)
###### (ie. FX spot, FX forwards, FX deposits and FX vol quotes)
###### Construct volatility surface using FinancePy library underneath, using polynomial interpolation
if run_example == 5 or run_example == 0:
    horizon_date = "03 Nov 2020"

    # Download the whole all market data for GBPUSD for pricing options (vol surface)
    md_request = MarketDataRequest(start_date=horizon_date,
                                   finish_date=horizon_date,
                                   data_source="bloomberg", cut="NYC",
                                   category="fx-vol-market",
                                   tickers=["USDJPY"],
                                   cache_algo="cache_algo_return")

    df = market.fetch_market(md_request)

    # Skip 3W/4M because this particular close (NYC) doesn"t have that in USDJPY market data
    tenors = ["ON", "1W", "2W", "1M", "2M", "3M", "6M", "9M", "1Y", "2Y", "3Y"]
    fx_vol_surface = FXVolSurface(market_df=df, tenors=tenors, asset="USDJPY")

    fx_vol_surface.build_vol_surface(horizon_date)

    # Note for unstable vol surface dates (eg. over Brexit date) you may need to increase tolerance in FinancePy
    # FinFXVolSurface.buildVolSurface method to get it to fill
    df_vol_dict = fx_vol_surface.extract_vol_surface()

    # Plot vol surface in strike space (all interpolated)
    # x_axis = strike - index
    # y_axis = tenor - columns
    # z_axis = implied vol - values
    chart.plot(df_vol_dict["vol_surface_strike_space"].iloc[:, ::-1],
               chart_type="surface",
               style=Style(title="Plotting volatility in strike space"))

    # Plot vol surface in delta space (exc market strangle strikes)
    chart.plot(df_vol_dict["vol_surface_delta_space_exc_ms"].iloc[:, ::-1],
               chart_type="surface",
               style=Style(title="Plotting in delta space"))



__author__ = 'saeedamen'  # Saeed Amen

#
# Copyright 2016-2020 Cuemacro - https://www.cuemacro.com / @cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

import datetime
from datetime import timedelta

import pandas as pd

class MarketUtil(object):

    def parse_date(self, date):
        if isinstance(date, str):

            date1 = datetime.datetime.utcnow()

            if date == "midnight":
                date1 = datetime.datetime(date1.year, date1.month, date1.day, 0, 0, 0)
            elif date == "decade":
                date1 = date1 - timedelta(days=365 * 10)
            elif date == "year":
                date1 = date1 - timedelta(days=365)
            elif date == "month":
                date1 = date1 - timedelta(days=30)
            elif date == "week":
                date1 = date1 - timedelta(days=7)
            elif date == "day":
                date1 = date1 - timedelta(days=1)
            elif date == "hour":
                date1 = date1 - timedelta(hours=1)
            else:
                # format expected 'Jun 1 2005 01:33', '%b %d %Y %H:%M'
                try:
                    date1 = datetime.datetime.strptime(date, '%b %d %Y %H:%M')
                except:
                    # ogger.warning("Attempted to parse date")
                    i = 0

                # format expected '1 Jun 2005 01:33', '%d %b %Y %H:%M'
                try:
                    date1 = datetime.datetime.strptime(date, '%d %b %Y %H:%M')
                except:
                    # logger.warning("Attempted to parse date")
                    i = 0

                try:
                    date1 = datetime.datetime.strptime(date, '%b %d %Y')
                except:
                    # logger.warning("Attempted to parse date")
                    i = 0

                try:
                    date1 = datetime.datetime.strptime(date, '%d %b %Y')
                except:
                    # logger.warning("Attempted to parse date")
                    i = 0
        else:
            date1 = pd.Timestamp(date)

        return pd.Timestamp(date1)


__author__ = 'saeedamen'  # Saeed Amen

#
# Copyright 2016-2021 Cuemacro - https://www.cuemacro.com / @cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

class MarketConstants(object):
    """Has various constants required for the finmarketpy project. These have been defined as static variables.

    """
    import platform

    ###### SETUP ENVIRONMENT VARIABLES ######
    plat = str(platform.platform()).lower()

    if 'linux' in plat: 
        generic_plat = 'linux'
    elif 'windows' in plat: 
        generic_plat = 'windows'
    elif ('darwin' in plat or 'macos' in plat): 
        generic_plat = 'mac'

    # "thread" or "multiprocessing" (experimental!) library to use when backtesting
    backtest_thread_technique = "multiprocessing"

    multiprocessing_library = 'multiprocess'  # 'multiprocessing_on_dill' or 'multiprocess' or 'multiprocessing' or 'pathos'

    # how many threads to use for backtesting (don't do too many on slow machines!)
    # also some data sources will complain if you start too many parallel threads to call data!
    # for some data providers might get better performance from 1 thread only!
    backtest_thread_no = {'linux': 8,
                          'windows' : 1,
                          'mac' : 8}

    hdf5_file_econ_file = "x"

    db_database_econ_file = ''
    db_server = '127.0.0.1'
    db_port = '27017'

    db_username = 'admin_root'
    db_password = 'TOFILL'

    write_engine = 'arctic'

    spot_depo_tenor = 'ON'

    currencies_with_365_basis =  ['AUD', 'CAD', 'GBP', 'NZD']

    # Whether to output additional fields related to calculation of total return indices
    output_calculation_fields = False

### FX Forwards ########################################################################################################
    fx_forwards_points_divisor_1 = ['IDR']
    fx_forwards_points_divisor_100 = ['JPY']
    fx_forwards_points_divisor_1000 = []

    # Forwards typically used for interpolation (note: eg. TN and SN are swaps)
    fx_forwards_tenor_for_interpolation = ["1W", "2W", "3W", "1M", "2M", "3M", "4M", "6M", "9M", "1Y", "2Y", "3Y", "5Y"]

    # What contract will we generally be trading?
    fx_forwards_trading_tenor = '1M'

    # When constructing total return index 'mult' or 'add'
    fx_forwards_cum_index = 'mult'

    # What is the point at which we roll?
    fx_forwards_roll_event = 'month-end' # 'month-end', 'quarter-end', 'year-end', 'delivery-date'

    # How many days before that point should we roll?
    fx_forwards_roll_days_before = 5

    # Typically when do we roll the contract?
    fx_forwards_roll_months = 1

### FX Options ########################################################################################################
    fx_options_points_divisor_100 = ['JPY']
    fx_options_points_divisor_1000 = []

    # Option tenors typically used for interpolation (note: eg. TN and SN are swaps)
    fx_options_tenor_for_interpolation = ["ON", "1W", "2W", "3W", "1M", "2M", "3M", "4M", "6M", "9M", "1Y"]#, "2Y", "3Y"]

    # What contract will we generally be trading?
    fx_options_trading_tenor = "1M"

    # When constructing total return index 'mult' or 'add'
    fx_options_cum_index = "mult"

    # For total return index use option price in base currency/for
    fx_options_index_premium_output = "pct-for"

    fx_options_index_strike = "atm"
    fx_options_index_contract_type = "european-call"

    fx_options_freeze_implied_vol = False

    # What is the point at which we roll?
    fx_options_roll_event = "expiry-date"  # 'month-end', 'expiry-date', 'no-roll'

    # How many days before that point should we roll?
    fx_options_roll_days_before = 5

    # Typically when do we roll the contract?
    fx_options_roll_months = 1

    # For fitting vol surface

    # 'CLARK5', 'CLARK', 'BBG', 'SABR' and 'SABR3'
    fx_options_vol_function_type = "CLARK5"
    fx_options_depo_tenor = "1M"

    # 'fwd-delta-neutral' or 'fwd-delta-neutral-premium-adj' or 'spot' or 'fwd'
    fx_options_atm_method = "fwd-delta-neutral-premium-adj"

    # 'fwd-delta' or 'fwd-delta-prem-adj' or 'spot-delta-prem-adj' or 'spot-delta'
    fx_options_delta_method = "spot-delta-prem-adj"
    fx_options_alpha = 0.5

    # 'pct-for' (in base currency pct) or 'pct-dom' (in terms currency pct)
    fx_options_premium_output = "pct-for"
    fx_options_delta_output = "pct-fwd-delta-prem-adj"

    # 'nelmer-mead' or 'nelmer-mead-numba' (faster but less accurate) or 'cg' (conjugate gradient tends to be slower, but more accurate)
    fx_options_solver = "nelmer-mead-numba"
    fx_options_pricing_engine = "financepy" # 'financepy' or 'finmarketpy'

    fx_options_tol = 1e-8

    override_fields = {}

    # Overwrite field variables with those listed in MarketCred or we can pass through an override_fields dictionary
    def __init__(self, override_fields={}):
        try:
            from finmarketpy.util.marketcred import MarketCred
            cred_keys = MarketCred.__dict__.keys()

            for k in MarketConstants.__dict__.keys():
                if k in cred_keys and '__' not in k:
                    setattr(MarketConstants, k, getattr(MarketCred, k))
        except:
            pass

        # Store overrided fields
        if override_fields == {}:
            override_fields = MarketConstants.override_fields
        else:
            MarketConstants.override_fields = override_fields

        for k in override_fields.keys():
            if '__' not in k:
                setattr(MarketConstants, k, override_fields[k])


__author__ = "saeedamen"

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""
Shows how to download market data from Quandl
"""


# loading data
import datetime

from chartpy import Chart, Style
from findatapy.market import Market, MarketDataGenerator, MarketDataRequest

from findatapy.util.loggermanager import LoggerManager

logger = LoggerManager().getLogger(__name__)

chart = Chart(engine="matplotlib")

market = Market(market_data_generator=MarketDataGenerator())

# Choose run_example = 0 for everything
# run_example = 1 - download

run_example = 0

###### fetch data from Quandl for BoE rate (using Bloomberg data)
if run_example == 1 or run_example == 0:
    # Monthly average of UK resident monetary financial institutions" 
    # (excl. Central Bank) sterling
    # Weighted average interest rate, other loans, new advances, on a 
    # fixed rate to private non-financial corporations (in percent)
    # not seasonally adjusted
    md_request = MarketDataRequest(
        start_date="01 Jan 2000",  # start date
        data_source="quandl",  # use Quandl as data source
        tickers=["Weighted interest rate"],
        fields=["close"],  # which fields to download
        vendor_tickers=["BOE/CFMBJ84"],  # ticker (Bloomberg)
        vendor_fields=["close"],  # which Bloomberg fields to download
        cache_algo="internet_load_return",
        quandl_api_key="TYPE YOUR KEY HERE")  # how to return data

    df = market.fetch_market(md_request)

    style = Style()

    style.title = "BoE weighted interest rate"
    style.scale_factor = 3
    style.file_output = "boe-rate.png"
    style.source = "Quandl/BoE"

    chart.plot(df, style=style)


__author__ = "saeedamen"

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


import pandas as pd
import numpy as np

from financepy.market.curves.discount_curve_flat import DiscountCurveFlat
from financepy.utils.date import Date

# Tested with financepy 0.310 only

from financepy.market.volatility.fx_vol_surface_plus import \
    FXVolSurfacePlus \
        as FinFXVolSurface  # So there is no clash with FXVolSurface from finmarketpy
from financepy.market.volatility.fx_vol_surface_plus import FinFXATMMethod
from financepy.market.volatility.fx_vol_surface_plus import \
    FinFXDeltaMethod
from financepy.market.volatility.fx_vol_surface_plus import vol_function
from financepy.market.volatility.fx_vol_surface_plus import \
    VolFunctionTypes

from financepy.utils.global_types import FinSolverTypes

from findatapy.util.dataconstants import DataConstants

from finmarketpy.curve.volatility.abstractvolsurface import AbstractVolSurface
from finmarketpy.util.marketconstants import MarketConstants
from finmarketpy.util.marketutil import MarketUtil

data_constants = DataConstants()
market_constants = MarketConstants()


class FXVolSurface(AbstractVolSurface):
    """Holds data for an FX vol surface and also interpolates vol surface,
    converts strikes to implied vols etc.

    """

    def __init__(self, market_df=None, asset=None, field="close",
                 tenors=market_constants.fx_options_tenor_for_interpolation,
                 vol_function_type=market_constants.fx_options_vol_function_type,
                 atm_method=market_constants.fx_options_atm_method,
                 delta_method=market_constants.fx_options_delta_method,
                 depo_tenor=market_constants.fx_options_depo_tenor,
                 solver=market_constants.fx_options_solver,
                 alpha=market_constants.fx_options_alpha,
                 tol=market_constants.fx_options_tol):
        """Initialises object, with market data and various market conventions

        Parameters
        ----------
        market_df : DataFrame
            Market data with spot, FX volatility surface, FX forwards and base depos

        asset : str
            Eg. "EURUSD"

        field : str
            Market data field to use

            default - "close"

        tenors : str(list)
            Tenors to be used (we need to avoid tenors, where there might be NaNs)

        vol_function_type : str
            What type of interpolation scheme to use
            default - "CLARK5" (also "CLARK", "BBG" and "SABR")

        atm_method : str
            How is the ATM quoted? Eg. delta neutral, ATMF etc.

            default - "fwd-delta-neutral-premium-adj"

        delta_method : str
            Spot delta, forward delta etc.

            default - "spot-delta"

        solver : str
            Which solver to use in FX vol surface calibration?

            default - "nelmer-mead"

        alpha : float
            Between 0 and 1 (default 0.5)
        """
        self._market_df = market_df
        self._tenors = tenors
        self._asset = asset
        self._field = field
        self._depo_tenor = depo_tenor
        self._market_util = MarketUtil()

        self._dom_discount_curve = None
        self._for_discount_curve = None
        self._spot = None

        self._value_date = None
        self._fin_fx_vol_surface = None
        self._df_vol_dict = None

        for_name_base = asset[0:3]
        dom_name_terms = asset[3:6]

        field = "." + field

        # CAREFUL: need to divide by 100 for depo rate, ie. 0.0346 = 3.46%
        self._forCCRate = market_df[
                              for_name_base + depo_tenor + field].values / 100.0  # 0.03460  # EUR
        self._domCCRate = market_df[
                              dom_name_terms + depo_tenor + field].values / 100.0  # 0.02940  # USD

        self._spot_history = market_df[asset + field].values
        self._atm_vols = market_df[
            [asset + "V" + t + field for t in tenors]].values
        self._market_strangle25DeltaVols = market_df[
            [asset + "25B" + t + field for t in tenors]].values
        self._risk_reversal25DeltaVols = market_df[
            [asset + "25R" + t + field for t in tenors]].values
        self._market_strangle10DeltaVols = market_df[
            [asset + "10B" + t + field for t in tenors]].values
        self._risk_reversal10DeltaVols = market_df[
            [asset + "10R" + t + field for t in tenors]].values

        if vol_function_type == "CLARK":
            self._vol_function_type = VolFunctionTypes.CLARK
        elif vol_function_type == "CLARK5":
            self._vol_function_type = VolFunctionTypes.CLARK5
        elif vol_function_type == "BBG":
            self._vol_function_type = VolFunctionTypes.BBG

        # Note: currently SABR isn"t fully implemented in FinancePy
        elif vol_function_type == "SABR":
            self._vol_function_type = VolFunctionTypes.SABR
        elif vol_function_type == "SABR3":
            self._vol_function_type = VolFunctionTypes.SABR3

        # What does ATM mean? (for most
        if atm_method == "fwd-delta-neutral":  # ie. strike such that a straddle would be delta neutral
            self._atm_method = FinFXATMMethod.FWD_DELTA_NEUTRAL
        elif atm_method == "fwd-delta-neutral-premium-adj":
            self._atm_method = FinFXATMMethod.FWD_DELTA_NEUTRAL_PREM_ADJ
        elif atm_method == "spot":  # ATM is spot
            self._atm_method = FinFXATMMethod.SPOT
        elif atm_method == "fwd":  # ATM is forward
            self._atm_method = FinFXATMMethod.FWD

        # How are the deltas quoted?
        if delta_method == "spot-delta":
            self._delta_method = FinFXDeltaMethod.SPOT_DELTA
        elif delta_method == "fwd-delta":
            self._delta_method = FinFXDeltaMethod.FORWARD_DELTA
        elif delta_method == "spot-delta-prem-adj":
            self._delta_method = FinFXDeltaMethod.SPOT_DELTA_PREM_ADJ
        elif delta_method == "fwd-delta-prem-adj":
            self._delta_method = FinFXDeltaMethod.FORWARD_DELTA_PREM_ADJ

        # Which solver to use in FX vol surface calibration
        if solver == "nelmer-mead":
            self._solver = FinSolverTypes.NELDER_MEAD
        elif solver == "nelmer-mead-numba":
            self._solver = FinSolverTypes.NELDER_MEAD_NUMBA
        elif solver == "cg":
            self._solver = FinSolverTypes.CONJUGATE_GRADIENT

        self._alpha = alpha
        self._tol = tol

    def build_vol_surface(self, value_date):
        """Builds the implied volatility surface for a particular value date and calculates the benchmark strikes etc.

        Before we do any sort of interpolation later, we need to build the implied_vol vol surface.

        Parameters
        ----------
        value_date : str
            Value date (need to have market data for this date)

        asset : str
            Asset name

        depo_tenor : str
            Depo tenor to use

            default - "1M"

        field : str
            Market data field to use

            default - "close"
        """

        self._value_date = self._market_util.parse_date(value_date)

        value_fin_date = self._findate(self._value_date)

        date_index = self._market_df.index == value_date

        # TODO: add whole rates curve
        dom_discount_curve = DiscountCurveFlat(value_fin_date,
                                               self._domCCRate[date_index])
        for_discount_curve = DiscountCurveFlat(value_fin_date,
                                               self._forCCRate[date_index])

        self._dom_discount_curve = dom_discount_curve
        self._for_discount_curve = for_discount_curve

        self._spot = float(self._spot_history[date_index][0])

        # New implementation in FinancePy also uses 10d for interpolation
        self._fin_fx_vol_surface = FinFXVolSurface(
            value_fin_date,
            self._spot,
            self._asset,
            self._asset[0:3],
            dom_discount_curve,
            for_discount_curve,
            self._tenors.copy(),
            self._atm_vols[date_index][
                0],
            self._market_strangle25DeltaVols[
                date_index][0],
            self._risk_reversal25DeltaVols[
                date_index][0],
            self._market_strangle10DeltaVols[
                date_index][0],
            self._risk_reversal10DeltaVols[
                date_index][0],
            self._alpha,
            atmMethod=self._atm_method,
            deltaMethod=self._delta_method,
            volatility_function_type=self._vol_function_type,
            finSolverType=self._solver,
            tol=self._tol)  # TODO add tol

    def calculate_vol_for_strike_expiry(self, K, expiry_date=None, tenor="1M"):
        """Calculates the implied_vol volatility for a given strike and tenor (or expiry date, if specified). The
        expiry date/broken dates are intepolated linearly in variance space.

        Parameters
        ----------
        K : float
            Strike for which to find implied_vol volatility

        expiry_date : str (optional)
            Expiry date of option

        tenor : str (optional)
            Tenor of option

            default - "1M"

        Returns
        -------
        float
        """

        if expiry_date is not None:
            expiry_date = self._findate(
                self._market_util.parse_date(expiry_date))
            try:
                return self._fin_fx_vol_surface.vol_from_strike_dt(
                    K, expiry_date)
            except:
                return self._fin_fx_vol_surface.volatility_from_strike_date(
                    K, expiry_date)
        else:
            try:
                tenor_index = self._get_tenor_index(tenor)
                return self.get_vol_from_quoted_tenor(K, tenor_index)
            except:
                pass

        return None

    def calculate_vol_for_delta_expiry(self, delta_call, expiry_date=None):
        """Calculates the implied_vol volatility for a given delta call and expiry date. The
        expiry date/broken dates are intepolated linearly in variance space.

        Parameters
        ----------
        delta_call : float
            Delta for the strike for which to find implied volatility

        expiry_date : str (optional)
            Expiry date of option

        Returns
        -------
        float
        """
        if expiry_date is not None:
            expiry_date = self._findate(
                self._market_util.parse_date(expiry_date))
            return self._fin_fx_vol_surface.volatilityFromDeltaDate(delta_call,
                                                                    expiry_date)

        return None

    def extract_vol_surface(self, num_strike_intervals=60, low_K_pc=0.95,
                            high_K_pc=1.05):
        """Creates an interpolated implied vol surface which can be plotted 
        (in strike space), and also in delta space for key strikes (ATM, 25d 
        call and put). Also for key strikes converts from delta to strike space.

        Parameters
        ----------
        num_strike_intervals : int
            Number of points to interpolate

        Returns
        -------
        dict
        """
        ## Modified from FinancePy code for plotting vol curves

        # columns = tenors
        df_vol_surface_strike_space = pd.DataFrame(
            columns=self._fin_fx_vol_surface._tenors)
        df_vol_surface_delta_space = pd.DataFrame(
            columns=self._fin_fx_vol_surface._tenors)

        # columns = tenors
        df_vol_surface_implied_pdf = pd.DataFrame(
            columns=self._fin_fx_vol_surface._tenors)

        # Conversion between main deltas and strikes
        df_deltas_vs_strikes = pd.DataFrame(
            columns=self._fin_fx_vol_surface._tenors)

        # ATM, 10d + 25d market strangle and 25d risk reversals
        df_vol_surface_quoted_points = pd.DataFrame(
            columns=self._fin_fx_vol_surface._tenors)

        # Note, at present we"re not using 10d strikes
        quoted_strikes_names = ["ATM", "STR_25D_MS", "RR_25D_P", "STR_10D_MS",
                                "RR_10D_P"]
        key_strikes_names = ["K_10D_P", "K_10D_P_MS", "K_25D_P", "K_25D_P_MS",
                             "ATM", "K_25D_C", "K_25D_C_MS", "K_10D_C",
                             "K_10D_C_MS"]

        # Get max/min strikes to interpolate (from the longest dated tenor)
        low_K = self._fin_fx_vol_surface._K_25D_P[-1] * low_K_pc
        high_K = self._fin_fx_vol_surface._K_25D_C[-1] * high_K_pc

        if num_strike_intervals is not None:
            # In case using old version of FinancePy
            try:
                implied_pdf_fin_distribution = self._fin_fx_vol_surface.implied_dbns(
                    low_K, high_K, num_strike_intervals)
            except:
                pass

        for tenor_index in range(0, self._fin_fx_vol_surface._num_vol_curves):

            # Get the quoted vol points
            tenor_label = self._fin_fx_vol_surface._tenors[tenor_index]

            atm_vol = self._fin_fx_vol_surface._atm_vols[tenor_index] * 100
            ms_25d_vol = self._fin_fx_vol_surface._mktStrangle25DeltaVols[
                             tenor_index] * 100
            rr_25d_vol = self._fin_fx_vol_surface._riskReversal25DeltaVols[
                             tenor_index] * 100
            ms_10d_vol = self._fin_fx_vol_surface._mktStrangle10DeltaVols[
                             tenor_index] * 100
            rr_10d_vol = self._fin_fx_vol_surface._riskReversal10DeltaVols[
                             tenor_index] * 100

            df_vol_surface_quoted_points[tenor_label] = pd.Series(
                index=quoted_strikes_names,
                data=[atm_vol, ms_25d_vol, rr_25d_vol, ms_10d_vol, rr_10d_vol])

            # Do interpolation in strike space for the implied vols (if intervals have been specified)
            strikes = []
            vols = []

            if num_strike_intervals is not None:
                K = low_K
                dK = (high_K - low_K) / num_strike_intervals

                for i in range(0, num_strike_intervals):
                    sigma = self.get_vol_from_quoted_tenor(K,
                                                           tenor_index) * 100.0
                    strikes.append(K)
                    vols.append(sigma)
                    K = K + dK

                df_vol_surface_strike_space[tenor_label] = pd.Series(
                    index=strikes, data=vols)

            try:
                df_vol_surface_implied_pdf[tenor_label] = pd.Series(
                    index=implied_pdf_fin_distribution[tenor_index]._x,
                    data=implied_pdf_fin_distribution[tenor_index]._densitydx)
            except:
                pass

            # Extract strikes for the quoted points (ie. 10d, 25d and ATM)
            key_strikes = []
            key_strikes.append(self._fin_fx_vol_surface._K_10D_P[tenor_index])
            key_strikes.append(
                self._fin_fx_vol_surface._K_10D_P_MS[tenor_index])
            key_strikes.append(self._fin_fx_vol_surface._K_25D_P[tenor_index])
            key_strikes.append(
                self._fin_fx_vol_surface._K_25D_P_MS[tenor_index])
            key_strikes.append(self._fin_fx_vol_surface._K_ATM[tenor_index])
            key_strikes.append(self._fin_fx_vol_surface._K_25D_C[tenor_index])
            key_strikes.append(
                self._fin_fx_vol_surface._K_25D_C_MS[tenor_index])
            key_strikes.append(self._fin_fx_vol_surface._K_10D_C[tenor_index])
            key_strikes.append(
                self._fin_fx_vol_surface._K_10D_C_MS[tenor_index])

            df_deltas_vs_strikes[tenor_label] = pd.Series(
                index=key_strikes_names, data=key_strikes)

            # Put a conversion between quoted deltas and strikes (eg. which is ATM in strike space, 25d call/put strikes)
            key_vols = []

            for K, name in zip(key_strikes, key_strikes_names):
                sigma = self.get_vol_from_quoted_tenor(K, tenor_index) * 100.0
                key_vols.append(sigma)

            df_vol_surface_delta_space[tenor_label] = pd.Series(
                index=key_strikes_names, data=key_vols)

        df_vol_dict = {}
        df_vol_dict["vol_surface_implied_pdf"] = df_vol_surface_implied_pdf
        df_vol_dict["vol_surface_strike_space"] = df_vol_surface_strike_space
        df_vol_dict["vol_surface_delta_space"] = df_vol_surface_delta_space
        df_vol_dict["vol_surface_delta_space_exc_ms"] = \
            df_vol_surface_delta_space[
                ~df_vol_surface_delta_space.index.str.contains("_MS")]
        df_vol_dict["vol_surface_quoted_points"] = df_vol_surface_quoted_points
        df_vol_dict["deltas_vs_strikes"] = df_deltas_vs_strikes

        self._df_vol_dict = df_vol_dict

        return df_vol_dict

    def get_vol_from_quoted_tenor(self, K, tenor, gaps=None):

        if not (isinstance(tenor, int)):
            tenor_index = self._get_tenor_index(tenor)
        else:
            tenor_index = tenor

        if gaps is None:
            gaps = np.array([0.1])

        params = self._fin_fx_vol_surface._parameters[tenor_index]
        t = self._fin_fx_vol_surface._texp[tenor_index]
        f = self._fin_fx_vol_surface._F0T[tenor_index]

        return vol_function(self._vol_function_type.value, params,
                            np.array([K]), gaps, f, K, t)

    def get_vol_strike_from_delta_tenor(self, call_delta, tenor=None,
                                        expiry_date=None):

        if tenor is not None:
            if not (isinstance(tenor, int)):
                tenor_index = self._get_tenor_index(tenor)
            else:
                tenor_index = tenor

            expiry_date = self._fin_fx_vol_surface._expiryDates[tenor_index]

        return self._fin_fx_vol_surface.volatilityFromDeltaDate(call_delta,
                                                                expiryDate=expiry_date)

    def get_atm_method(self):
        return self._atm_method

    def get_delta_method(self):
        return self._delta_method

    def get_all_market_data(self):
        return self._market_df

    def get_spot(self):
        return self._spot

    def get_atm_strike(self, tenor=None):
        return self._df_vol_dict["deltas_vs_strikes"][tenor]["ATM"]

    def get_25d_call_strike(self, tenor=None):
        return self._df_vol_dict["deltas_vs_strikes"][tenor]["K_25D_C"]

    def get_25d_put_strike(self, tenor=None):
        return self._df_vol_dict["deltas_vs_strikes"][tenor]["K_25D_P"]

    def get_10d_call_strike(self, tenor=None):
        return self._df_vol_dict["deltas_vs_strikes"][tenor]["K_10D_C"]

    def get_10d_put_strike(self, tenor=None):
        return self._df_vol_dict["deltas_vs_strikes"][tenor]["K_10D_P"]

    def get_25d_call_ms_strike(self, tenor=None):
        return self._df_vol_dict["deltas_vs_strikes"][tenor]["K_25D_C_MS"]

    def get_25d_put_ms_strike(self, tenor=None):
        return self._df_vol_dict["deltas_vs_strikes"][tenor]["K_25D_P_MS"]

    def get_10d_call_ms_strike(self, expiry_date=None, tenor=None):
        return self._df_vol_dict["deltas_vs_strikes"][tenor]["K_10D_C_MS"]

    def get_10d_put_ms_strike(self, expiry_date=None, tenor=None):
        return self._df_vol_dict["deltas_vs_strikes"][tenor]["K_10D_P_MS"]

    def get_atm_quoted_vol(self, tenor):
        """The quoted ATM vol from the market (ie. which has NOT been obtained from build vol surface)

        Parameters
        ----------
        tenor : str
            Tenor

        Returns
        -------
        float
        """
        return self._atm_vols[self._market_df.index == self._value_date][0][
            self._get_tenor_index(tenor)]

    def get_atm_vol(self, tenor=None):
        return self._df_vol_dict["vol_surface_delta_space"][tenor]["ATM"]

    def get_25d_call_vol(self, tenor=None):
        return self._df_vol_dict["vol_surface_delta_space"][tenor]["K_25D_C"]

    def get_25d_put_vol(self, tenor=None):
        return self._df_vol_dict["vol_surface_delta_space"][tenor]["K_25D_P"]

    def get_25d_call_ms_vol(self, tenor=None):
        return self._df_vol_dict["vol_surface_delta_space"][tenor][
            "K_25D_C_MS"]

    def get_25d_put_ms_vol(self, tenor=None):
        return self._df_vol_dict["vol_surface_delta_space"][tenor][
            "K_25D_P_MS"]

    def get_10d_call_vol(self, tenor=None):
        return self._df_vol_dict["vol_surface_delta_space"][tenor]["K_10D_C"]

    def get_10d_put_vol(self, tenor=None):
        return self._df_vol_dict["vol_surface_delta_space"][tenor]["K_10D_P"]

    def get_10d_call_ms_vol(self, tenor=None):
        return self._df_vol_dict["vol_surface_delta_space"][tenor][
            "K_10D_C_MS"]

    def get_10d_put_ms_vol(self, tenor=None):
        return self._df_vol_dict["vol_surface_delta_space"][tenor][
            "K_10D_P_MS"]

    def get_dom_discount_curve(self):
        return self._dom_discount_curve

    def get_for_discount_curve(self):
        return self._for_discount_curve

    def plot_vol_curves(self):
        if self._fin_fx_vol_surface is not None:
            self._fin_fx_vol_surface.plotVolCurves()

    def _findate(self, timestamp):

        return Date(timestamp.day, timestamp.month, timestamp.year,
                    hh=timestamp.hour, mm=timestamp.minute,
                    ss=timestamp.second)



from setuptools import setup, find_packages

long_description = """finmarketpy is a Python based library that enables you to 
analyze market data and also to backtest trading strategies using a simple to 
use API, which has prebuilt templates for you to define backtest."""

setup(name="finmarketpy",
      version="0.11.14",
      description="finmarketpy is a Python based library for backtesting trading strategies",
      author="Saeed Amen",
      author_email="saeed@cuemacro.com",
      license="Apache 2.0",
      long_description=long_description,
      keywords=["trading", "markets", "currencies", "pandas", "data",
                "Bloomberg", "tick", "stocks", "equities"],
      url="https://github.com/cuemacro/finmarketpy",
      packages=find_packages(),
      include_package_data=True,
      install_requires=["pandas",
                        "twython",
                        "pytz",
                        "requests",
                        "numpy",
                        "multiprocess",
                        "seasonal",
                        "scikit-learn",
                        "matplotlib",
                        "numba",
                        "financepy==0.310"],
      zip_safe=False)



__author__ = "saeedamen"

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""
Shows how to use finmarketpy to create total return indices for FX spot (ie. 
calculates spot returns + carry returns)
"""

import pandas as pd

# For plotting
from chartpy import Chart, Style

# For loading market data
from findatapy.market import Market, MarketDataGenerator, MarketDataRequest
from findatapy.timeseries import Calculations, Calendar

from findatapy.util.loggermanager import LoggerManager

logger = LoggerManager().getLogger(__name__)

chart = Chart(engine='plotly')
market = Market(market_data_generator=MarketDataGenerator())
calculations = Calculations()

# Choose run_example = 0 for everything
# run_example = 1 - get forwards prices for AUDUSD interpolated for an odd date/broken date
# run_example = 2 - get implied deposit rate

run_example = 0

from finmarketpy.curve.rates.fxforwardspricer import FXForwardsPricer

###### Value forwards for AUDUSD for odd days
if run_example == 1 or run_example == 0:

    cross = 'AUDUSD'
    fx_forwards_tenors = ['1W', '2W', '3W', '1M']

    fx_forwards_to_print = ['1W', '2W']

    # Get AUDUSD data for spot, forwards + depos
    md_request = MarketDataRequest(start_date='01 Jan 2020', finish_date='01 Feb 2020',
                                   data_source='bloomberg', cut='NYC', category='fx-forwards-market',
                                   tickers=cross,
                                   cache_algo='cache_algo_return', fx_forwards_tenor=fx_forwards_tenors,
                                   base_depos_currencies=[cross[0:3], cross[3:6]])

    market_df = market.fetch_market(md_request=md_request)

    fx_forwards_price = FXForwardsPricer()

    delivery_dates = Calendar().get_delivery_date_from_horizon_date(market_df.index, "8D", cal=cross)
    interpolated_forwards_df = fx_forwards_price.price_instrument(cross, market_df.index, delivery_dates,
        market_df=market_df, fx_forwards_tenor_for_interpolation =['1W', '2W'])

    interpolated_forwards_df[cross + ".delivery"] = delivery_dates.values

    print(interpolated_forwards_df)

    market_cols = [cross + ".close"]

    for f in fx_forwards_to_print:
        market_cols.append(cross + f + '.close')

    print(market_df[market_cols])

if run_example == 2 or run_example == 0:

    cross = 'AUDUSD'
    fx_forwards_tenors = ['1W', '2W', '3W', '1M']
    fx_forwards_to_print = ['1M']

    # Get AUDUSD data for forwards (spot, forwards points and base depos)
    md_request = MarketDataRequest(start_date='01 Jan 2020', finish_date='01 Feb 2020',
                                   data_source='bloomberg', cut='NYC', category='fx-forwards-market',
                                   tickers=cross,
                                   cache_algo='cache_algo_return', fx_forwards_tenor=fx_forwards_tenors,
                                   base_depos_tenor=fx_forwards_tenors,
                                   base_depos_currencies=[cross[0:3], cross[3:6]])

    market_df = market.fetch_market(md_request=md_request)

    fx_forwards_price = FXForwardsPricer()

    implied_depo_df = fx_forwards_price.calculate_implied_depo(cross, 'AUD', market_df=market_df,
                               fx_forwards_tenor=fx_forwards_tenors,
                               depo_tenor='1M')

    print(implied_depo_df)

    market_cols = [cross + ".close"]

    for f in fx_forwards_to_print:
        market_cols.append(cross + f + '.close')

    market_cols.append("USD1M.close")
    print(market_df[market_cols])

    # Download implied deposits (via FX forwards) for base currency (AUD) from Bloomberg
    # Note: some implied deposits tenors might have no data in it
    tickers = []

    for t in fx_forwards_tenors:
        tickers.append(cross[0:3] + "I" + t)

    md_request = MarketDataRequest(start_date='01 Jan 2020', finish_date='01 Feb 2020',
                                   data_source='bloomberg', cut='NYC',
                                   tickers=tickers,
                                   vendor_tickers= [x + " CMPN Curncy" for x in tickers],
                                   cache_algo='cache_algo_return')

    implied_depo_bbg_df = market.fetch_market(md_request=md_request)

    chart.plot(calculations.join([implied_depo_bbg_df, implied_depo_df]), how='outer')




__author__ = "saeedamen"  # Saeed Amen

#
# Copyright 2016-2020 Cuemacro - https://www.cuemacro.com / @cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

"""
Shows how to use finmarketpy to create total return indices for FX forwards with appropriate roll rules
"""

import pandas as pd

# For plotting
from chartpy import Chart, Style

# For loading market data
from findatapy.market import Market, MarketDataGenerator, MarketDataRequest
from findatapy.timeseries import Calculations

from findatapy.util.loggermanager import LoggerManager

logger = LoggerManager().getLogger(__name__)

from finaddpy.market import CachedMarketDataGenerator as MarketDataGenerator

chart = Chart(engine="plotly")
market = Market(market_data_generator=MarketDataGenerator())
calculations = Calculations()

# Choose run_example = 0 for everything
# run_example = 1 - creating USDBRL total return index rolling forwards and compare with BBG indices
# run_example = 2 - creating AUDJPY (via AUDUSD and JPYUSD) total return index rolling forwards & compare with BBG indices

run_example = 0

from finmarketpy.curve.fxforwardscurve import FXForwardsCurve

###### Create total return indices plot for USDBRL using forwards
# We shall be using USDBRL 1M forward contracts and rolling them 5 business days before month end
if run_example == 1 or run_example == 0:
    cross = "USDIDR"

    # Download more tenors
    fx_forwards_tenors = ["1M", "2M", "3M"]

    # Get USDBRL data for spot, forwards + depos
    md_request = MarketDataRequest(start_date="02 Jan 2007", finish_date="01 Jul 2007",
                                   data_source="bloomberg", cut="NYC", category="fx-forwards-market",
                                   tickers=cross,
                                   fx_forwards_tenor=fx_forwards_tenors,
                                   base_depos_currencies=[cross[0:3], cross[3:6]],
                                   cache_algo="cache_algo_return")

    # In case any missing values fill down (particularly can get this for NDFs)
    df_market = market.fetch_market(md_request=md_request).fillna(method="ffill")

    fx_forwards_curve = FXForwardsCurve()

    # Let"s trade a 1M forward, and we roll 5 business days (based on both base + terms currency holidays)
    # before month end
    df_cuemacro_tot_1M = fx_forwards_curve.construct_total_return_index(cross, df_market,
                                     fx_forwards_trading_tenor="1M",
                                     roll_days_before=5,
                                     roll_event="month-end",
                                     roll_months=1,
                                     fx_forwards_tenor_for_interpolation=fx_forwards_tenors,
                                     output_calculation_fields=True)

    df_cuemacro_tot_1M.columns = [x.replace("forward-tot", "forward-tot-1M-cuemacro") for x in df_cuemacro_tot_1M.columns]

    # Get spot data
    md_request.abstract_curve = None
    md_request.category = "fx"

    df_spot = market.fetch_market(md_request=md_request)
    df_spot.columns = [x + "-spot" for x in df_spot.columns]

    # Get Bloomberg calculated total return indices (for spot)
    md_request.category = "fx-tot"

    df_bbg_tot = market.fetch_market(md_request)
    df_bbg_tot.columns = [x + "-bbg" for x in df_bbg_tot.columns]

    # Get Bloomberg calculated total return indices (for 1M forwards rolled)
    md_request.category = "fx-tot-forwards"

    df_bbg_tot_forwards = market.fetch_market(md_request)
    df_bbg_tot_forwards.columns = [x + "-bbg" for x in df_bbg_tot_forwards.columns]

    # Combine into a single data frame and plot, we note that the Cuemacro constructed indices track the Bloomberg
    # indices relatively well (both from spot and forwards). Also note the large difference with spot indices
    # CAREFUL to fill down, before reindexing because forwards indices are likely to have different publishing dates
    df = calculations.join([pd.DataFrame(df_cuemacro_tot_1M[cross + "-forward-tot-1M-cuemacro.close"]),
                                         df_bbg_tot, df_spot, df_bbg_tot_forwards], how="outer").fillna(method="ffill")

    df = calculations.create_mult_index_from_prices(df)

    chart.plot(df)
    